try:
  import __init__ as sx #starfishX Production #__init__ debug Mode
except:
  import starfishX as sx

def elderSafezone(df,timeframe=10):
 df["DNPen"] = (df["Low"]-df["Low"].shift(-1)).shift(1)
 df.loc[df["DNPen"]<0,"DNPen"]=0
 df["DSUM"] = df["DNPen"].rolling(window=10).sum()
 df["DPENYN"] = 1
 df.loc[df["Low"]<df["Low"].shift(-1),"DPENYN"]=0
 df["DPENYN"] = df["DPENYN"].shift(1)
 df["DNNUM"] = df["DPENYN"].rolling(window=10).sum()
 df["DNAvg"] = (df["DSUM"]/df["DNNUM"]).round(2)
 df["shortStop"] = df["Low"]-(2*df["DNAvg"])
 df["shortStop"] = df["shortStop"].shift(1)
 df["DProtected"] = df["shortStop"].rolling(window=3).max()

 df["HNPen"] = df["High"]-df["High"].shift(1)
 df.loc[df["HNPen"]<0,"HNPen"]=0
 df["HPNUM"] = df["HNPen"].rolling(window=10).sum()
 df["HPENYN"]=1
 df.loc[df["High"]>df["High"].shift(-1),"HPENYN"]=0
 df["HPENYN"] = df["HPENYN"].shift(1)
 df["HNNUM"] = df["HPENYN"].rolling(window=10).sum()
 df["HNAvg"] = (df["HPNUM"]/df["HNNUM"]).round(2)
 df["LongStop"] = df["High"]+(2*df["HNAvg"])
 df["LongStop"] = df["LongStop"].shift(1)
 df["HProtected"] = df["LongStop"].rolling(window=3).min()

 return df[["DProtected","HProtected","Close"]]  


# P/E Proxy หาตัวแทนของ P/E 
import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np

def getPEProxyProcess(symbol):  
 '''
 หา PE ของหุ้น  
 parameter : รายชื่อหุ้น symbol

 return  
 - PE บริษัท
 '''  
 symbol = symbol.replace("&","%26") 
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
 url = "https://www.set.or.th/set/factsheet.do?symbol="+symbol+"&ssoPageId=3&language=th&country=TH"
 r = requests.post(url,headers=headers, timeout=5)
 soup = BeautifulSoup(r.content, "lxml")  

 ct = soup.find_all("table",{"class":"table-factsheet-padding0"})  

 if(len(ct)==0):
    pass#return 0

 #### หา index #### content อยู่ช่องไหน
 i_index = 0
 for i in ct:
  if("มูลค่าหลักทรัพย์ตามราคาตลาด" in i.text):
      m_index = i_index
      break
  i_index+=1
 
 #######
 tr = ct[m_index].find_all("tr")
   
 for i in tr:
  td = i.find_all("td")
  if("มูลค่าหลักทรัพย์ตามราคาตลาด" in td[0].text):
      try:
       marketCap  = float(td[1].text.replace(",",""))
      except: 
       marketCap  = np.NaN  
  if("P/E" in td[0].text):
      try:
       PE  = float(td[1].text.replace(",",""))
      except: 
       PE  = np.NaN  
      
      Earning = marketCap/PE  
 #######################################

 return marketCap,Earning

def upper(n):
 return n.upper()

def getPEProxy(symbollist):
 '''
 Symbol List รายชื่อของหุ้น เช่น ["ADVANC","DTAC","TRUE"]
 return PE,log
 '''
 marketCap = []
 earning = []
 pe_avg = []
 if(type(symbollist)==str):
    symbollist = [symbollist]
    
 for i in symbollist:
    m,e = getPEProxyProcess(i)
    marketCap.append(m)
    earning.append(e)
 
 ########
 df = pd.DataFrame({"Market Cap.":marketCap,"Earning":earning},index=symbollist) 
 df["P/E"] = df["Market Cap."]/df["Earning"]
 PEProxy = df["Market Cap."].sum()/df["Earning"].sum()

 df.index = df.index.map(upper)
 return PEProxy,df  


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

from matplotlib.pyplot import text

def trendMA_YMF(symbol,start,engine,plot=True):
  df = engine.getPrice(symbol,start=start)
  df['MA50'] = df[[symbol]].rolling(window=50).mean() 
  df['MA150'] = df[[symbol]].rolling(window=150).mean() 
  df['MA200'] = df[[symbol]].rolling(window=200).mean() 
  
  if(plot):
    df.plot(figsize=(15,6))
    plt.show()

  k = df[(df[symbol]>df['MA50']) & (df['MA50']>df['MA150']) & (df['MA150']>df['MA200'])]
    
  if(len(k)==0):
    return False,'fail condition (1),(2),(4) และ (8)'

  if(df.index[-1] == k.index[-1]):
    return True ,'pass condition (1),(2),(4) และ (8) : close > MA50 , MA50 > MA150 , MA150 > MA200'
  else:
   return False,'fail condition (1),(2),(4) และ (8)'

def trendMA_Year(symbol,start,engine,month=3,plot=True):
  
  df = engine.getPrice(symbol,start=start)
  df['MA200'] = df[[symbol]].rolling(window=200).mean()
  
  lastDay = df.index[-1]
  
  timeofmonth = month*22

  k = df[['MA200']].tail(timeofmonth).dropna()
  slope, intercept = np.polyfit(np.arange(0,len(k)), k['MA200'], 1)
  abline_values = [((slope * i) + intercept) for i in range(len(k['MA200']))]

  abline_values = np.array(abline_values)

  dt = df[['MA200']].tail(timeofmonth).dropna()
  #plt.plot(dt.index, dt['SMA200'], '--')
  LastDaySL = (lastDay + pd.Timedelta('20 day'))
  
  
  if(plot):
    plt.rcParams["figure.figsize"] = (15,6)
    
    df[[symbol,'MA200']].plot(figsize=(15,6))    
    plt.plot(dt.index, abline_values*0.95, 'w--',linewidth=2,label='Slope MA200 ('+str(month)+'Months)')
    text(LastDaySL,abline_values[-1]*0.95, 'Slope MA200 ('+str(month)+'Months)', 
         rotation=0,horizontalalignment='left')
    plt.legend()
    plt.show()

 
  if(slope>0):
    return True,'pass slope(+) pass condition (3)' + 'slope :'+str(slope)
  else:
    return False,'fail slope(-) fail condition (3)'+ 'slope :'+str(slope)  


def check52WeekHighLow(symbol,start,engine,plot=True):
    df = engine.getPrice(symbol,start=start)
    
    if(len(df)<252):
        return False,'log : not enough information'
    df['52W High'] = df[[symbol]].rolling(window=252).max() 
    df['52W Low'] = df[[symbol]].rolling(window=252).min() 
    
    price52WeekH = df.iloc[-1]['52W High']
    LastPrice = df.iloc[-1][symbol]
    lastDay = df.index[-1]
    
    price52WeekL = df.iloc[-1]['52W Low']
    
    pctD_From_week52H = (LastPrice - price52WeekH )/price52WeekH
    pctD_From_week52H = round(pctD_From_week52H*100,2)
    
    pctD_From_week52L = (LastPrice - price52WeekL)/price52WeekL
    pctD_From_week52L = round(pctD_From_week52L*100,2)
    #print('x',LastPrice,price52WeekL,pctD_From_week52L)

    if(plot):
      #บวกวันเพิ่ม เพื่อให้การแสดงผลไม่ติดเกินไปเท่านั้น
      lastDay_PD = lastDay+pd.Timedelta('5 day')
      lastDay_PD2 = lastDay+pd.Timedelta('20 day')
    
      df.plot(figsize=(15,6),title=symbol+' 52Week High & Low')
      
      plt.vlines(x=lastDay_PD,linestyle='--', ymin=LastPrice, 
               ymax=price52WeekH, color='green', zorder=2)

      text(lastDay_PD,LastPrice, 'Gap '+str(pctD_From_week52H)+'% / w52H', 
             rotation=90,horizontalalignment='left')
      
        
      plt.vlines(x=lastDay_PD,linestyle='--', ymin=price52WeekL, 
               ymax=LastPrice, color='red', zorder=2)
    
      text(lastDay_PD2,price52WeekL, 'Gap '+str(pctD_From_week52L)+'% / w52L', 
         rotation=90,horizontalalignment='left')
        
      
      dayX1 = lastDay_PD-pd.Timedelta('5 day')
      dayX2 = lastDay_PD+pd.Timedelta('5 day')
      plt.hlines(y=LastPrice,xmin=dayX1,xmax=dayX2, color='r', linestyle='--')
    
    c5 = False
    if(pctD_From_week52H > -20): #อย่าลงมาต่ำกว่าจุดสูงสุดเกิน 20%
        c5 = True
        logC5 = 'pass condition (5) ,last price '+ str(pctD_From_week52H)+'%'
    else:
        logC5 = 'fail condition (5) ,last price ' + str(pctD_From_week52H)+'%'
    
    c6 = False
    if(pctD_From_week52L > 20): #ราคาปัจจุบันอยู่เหนือกว่าราคา 52W Log เยอะๆ %
        c6 = True
        logC6 = 'pass condition (6) ,last price '+ str(pctD_From_week52L)+'%'
    else:
        logC6 = 'fail condition (6) ,last price ' + str(pctD_From_week52L)+'%'
        
    if(c5==True and c6==True):
       return True,logC5+' and '+logC6
    else:
       return False,logC5+' and '+logC6

def ROC_Basket(basket,start,engine):
  pct_change = []
  close_price = []
  close_start = []
  for i in basket:
    df = engine.getPrice(i,start)
    k = (df.tail(1)[i].values[0]/df.head(1)[i].values[0])-1
    pct_change.append(round(k*100,2))
    close_price.append(round(df.tail(1)[i].values[0],2))
    close_start.append(round(df.head(1)[i].values[0],2))  
    print('.',end='')
  print('done')

  df = pd.DataFrame({'symbol':basket,'close_start':close_start,'close_end':close_price,'pct_chg':pct_change})
  df = df.set_index('symbol')
  df.sort_values('pct_chg',ascending=False)

  plt.figure(figsize=(20,6))

  df = df.sort_values('pct_chg')
  color = np.where(df.index == 'SET','gold',  
                np.where(df.pct_chg>df[df.index=='SET']['pct_chg'].values[0],'green','red') 
                )
  ax = plt.bar(x=df.index,height=df['pct_chg'],color=color)

  df['no'] = np.arange(0,len(df))
  k = df[df.index=='SET'].no[0]

  plt.title('Rate of Change')
  plt.annotate("SET INDEX", 
            xy=(k, 10), xytext=(k, 50), arrowprops={"arrowstyle":"->", "color":"gray"})
  plt.xticks(rotation=90)
  plt.show
  return df


import pandas as pd
import difflib 

def ShareholdersNow(keyword):
 '''
    keyword : str
    คืนค่ารายชื่อผู้ถือหุ้นที่เทียบกับปิดสมุดล่าสุด
 '''
 df = pd.read_csv("ShareholdersTB.csv")
 df["Date"] = pd.to_datetime(df["Date"],format='%Y-%m-%d')
 df["NameSer"] = df["Name"].str.lower() 

 keyword = keyword.lower()
 df["Date"] = pd.to_datetime(df["Date"],format='%Y-%m-%d')
 df1 = df[df["NameSer"].str.contains(keyword)].groupby(["Symbol"]).max()
    
 df2 = pd.merge(df,df1,on=["Symbol"]).groupby("Symbol")[["Date_x"]].max()
 df2.columns = ["Date"]   
    
 df3 = pd.merge(df,df2,on=["Symbol","Date"])
 df4 = df3[df3["NameSer"].str.contains(keyword)]  
 df4 = df4[["Symbol","Name","Share","Percent","Date","Type"]]   
 return df4.set_index("Symbol")


#fix bug การ merge fuzzy
def get_close(x,xcol_c):
    if(type(x)==str): 
     tmp = difflib.get_close_matches(x,xcol_c)
     if(len(tmp)>0):
        return tmp[0]
     else:
        return x

def ShareholdersHist(symbol):
  '''
    symbol : str
    คืนค่ารายชื่อผู้ถือหุ้นที่เทียบแต่ละวันของการปิดสมุดล่าสุด
  ''' 
  #โหลดข้อมูลพื้นฐานทั้งหมดมาก่อน
  df = pd.read_csv("ShareholdersTB.csv")
  df["Date"] = pd.to_datetime(df["Date"],format='%Y-%m-%d') 
    
  #หาวันที่ปิดสมุด
  symbol = symbol.upper()#"JMART"
  date_hist = df[df["Symbol"]==symbol].groupby("Date")  \
           [["Date","Type"]].min().sort_index(ascending=True)
  
  
  #merge data แต่ละของวันที่ปิดสมุดเข้าด้วยกัน
  hist_Name  = []
  hist_Share = []
  hist_Percent  = []
  rp = pd.DataFrame()

  for i in range(len(date_hist)):
    dt_label = date_hist.iloc[[i]]["Date"][[0]].dt.strftime('%Y-%m-%d')[0]
    
    hist_Name.append("Name-"+dt_label)  
    hist_Share.append("Share-"+dt_label)
    hist_Percent.append("Percent-"+dt_label) 
    
    df_tmp = df[(df["Symbol"]==symbol) & (df["Date"]==dt_label) ][["Name","Share","Percent"]]
    df_tmp.columns = ["Name-"+dt_label,"Share-"+dt_label,"Percent-"+dt_label]  
    df_tmp = df_tmp.sort_values("Share-"+dt_label,ascending=False) 
   
    #df_tmp['Name-'+dt_label] = df_tmp['Name-'+dt_label].str.strip()
    #df_tmp['Name-'+dt_label] = df_tmp['Name-'+dt_label].str.replace(' ','@')
    #df_tmp['Name-'+dt_label] = df_tmp['Name-'+dt_label].str.replace('  ','@')
    #df_tmp['Name-'+dt_label] = df_tmp['Name-'+dt_label].str.replace('@','')
    #print(df_tmp['Name-'+dt_label])

    if(i==0):
      rp = df_tmp
    else:   
      #return rp,df_tmp
      #k = rp[rp.columns[0]].map(lambda x:get_close(x,df_tmp[df_tmp.columns[0]]) )
      #print(k)
      #if(i==2):
      #  return rp,df_tmp
      rp[rp.columns[0]] = rp[rp.columns[0]].map(lambda x:get_close(x,df_tmp[df_tmp.columns[0]]) )
      rp = rp.merge(df_tmp, left_on=hist_Name[i-1], right_on=hist_Name[i],how='outer')  
    
  #เรียงข้อมูลล่าสุด
  tmp = rp.sort_values(rp.columns[-1],ascending=False)
  

  #สร้างรายชื่อผู้ถือหุ้นทั้งหมด
  holdername = []
  for i in range(len(tmp.columns)):
    if("Name" in tmp.columns[i]):
     if(len(holdername)==0): #first columns
       holdername = tmp[tmp.columns[i]].values.copy()
    
    col = tmp.columns[i]
    for j in range(len(tmp[tmp.columns[0]])):   
     newitem = tmp[[col]].iloc[j].values[0]
     if(type(newitem)==str):  
       holdername[j] = newitem   
    
  tmp["Name"] = holdername  
  
    
  #เอาเฉพาะชื่อ คอลัมน์ที่ต้องการ
  c = ["Name"]
  for i in zip(hist_Share,hist_Percent):
    c.append(i[0])
    c.append(i[1])

  #เรียง index ใหม่
  tmp['no'] = (list(range(1,len(tmp[c])+1)))
  tmp = tmp.set_index("no")

  #กำหนดรูปแบบ float_format
  pd.options.display.float_format = '{:,.2f}'.format 
    
  #คืนค่าทั้งหมด
  return tmp[c] 


try:
    #ลำดับมีผลพยายามโหลดที่ ใน folder ที่กำลัง Implement ก่อน
    #import config as con 
    from starfishX import config as con 
except:
    import starfishX.config as con

prefix = "starfishX."

if(con.__mode__=="debug"):
    prefix = ""

import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np

#import squarify
import matplotlib
import matplotlib.pyplot as plt
#%config InlineBackend.figure_format ='retina'
from pylab import rcParams


exec("import "+prefix+"squarify")

def getTotalShare(symbol):
 url = "https://www.set.or.th/set/factsheet.do?symbol="+symbol+"&ssoPageId=3&language=th&country=TH"
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
 r = requests.get(url,headers,verify=False)
 soup = BeautifulSoup(r.content, "lxml")
 
 indexData = 11
    
 data = soup.find_all("table",{"class":"table-factsheet-padding0"})[indexData] #[2].find_all("tr") 
 #print(data.find_all("td"))
 
 if(not ("จำนวนหุ้นจดทะเบียน" in data.text)):
 #if(data.find_all("td")[0].text!='งบแสดงฐานะการเงิน (ลบ.)'):   
    indexData = 12
    data = soup.find_all("table",{"class":"table-factsheet-padding0"})[indexData]
 
 if(not ("จำนวนหุ้นจดทะเบียน" in data.text)):
 #if(data.find_all("td")[0].text!='งบแสดงฐานะการเงิน (ลบ.)'):   
    indexData = 13
    data = soup.find_all("table",{"class":"table-factsheet-padding0"})[indexData]
    
 if(not ("จำนวนหุ้นจดทะเบียน" in data.text)):
 #if(data.find_all("td")[0].text!='งบแสดงฐานะการเงิน (ลบ.)'):   
    indexData = 14
    data = soup.find_all("table",{"class":"table-factsheet-padding0"})[indexData]
    
 if(not ("จำนวนหุ้นจดทะเบียน" in data.text)):
 #if(data.find_all("td")[0].text!='งบแสดงฐานะการเงิน (ลบ.)'):   
    indexData = 15
    data = soup.find_all("table",{"class":"table-factsheet-padding0"})[indexData]   
 

 #print(indexData)
 p = 0
 totalShare = 0
 #print(data)
 for i in data.find_all("td"):
  if("จำนวนหุ้นจดทะเบียน" in i.text):
     p = 1
  else:
     if(p==1):
        totalShare = i.text.replace(",","")
        totalShare = float(totalShare)
        p = 0
        break
 return totalShare  



def marketValueDaily(marketype,sector="main"):
 """
 arg1 : indexMarket เช่น  sx.indexMarket.SET
 
 arg2 : sector จะใช้เฉพาะเมื่อ indexMarket เป็น SET เท่านั้น นอกนั้นไม่ต้องส่งค่านี้

 """ 
 rcParams['figure.figsize'] = 10, 10
 font = {'family' : 'normal','size': 10}
 matplotlib.rc('font', **font)
 
 typemarket = marketype.value
 if(typemarket=="SET" or typemarket=="mai"):
   url = "https://marketdata.set.or.th/mkt/sectorialindices.do?market="+typemarket+"&language=th&country=TH"
   if(typemarket=="mai" and sector!="main"):
     print("index not need Parameter 'sector' ")  
     return 0  
 else:
   url = "https://marketdata.set.or.th/mkt/sectorquotation.do?sector="+typemarket+"&language=th&country=TH"   
   if(sector!="main"):
      print("index not need Parameter 'sector' ")  
      return 0
    
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
 r = requests.get(url,headers,verify=False)
 soup = BeautifulSoup(r.content, "lxml")

 data = soup.find_all("tbody")[2].find_all("tr") 
 #print(data)
 secLB_DS =[]
 secValue_DS = []
 for i in data:
  try:
    if(i.get("class")[0]=="industry"):
      #print(1)
      info = i.find_all("td")  
      sec = info[0].text.strip()
      value = info[5].text.strip().replace(",","") 
      #print(sec,value)  
      if(sector=="main"):
        secLB_DS.append(sec)  
        secValue_DS.append(float(value)) 
  except:
      #print(2)
      kIndex = 5
      if(typemarket!="mai" and typemarket!="SET"): 
         kIndex = kIndex+6

      info = i.find_all("td")
      sec = info[0].text.strip()
      value = info[kIndex].text.strip() .replace(",","") 
      #print("\t",sec,value)
     
      if(sector=="sub" or typemarket=="mai" or typemarket!="SET"):  
       secLB_DS.append(sec)  
       secValue_DS.append(float(value)) 
      
 #print(secValue_DS)

 c = (sum(secValue_DS)**2)**(1/2)
 col = np.array(secValue_DS)/c
 cm = plt.cm.get_cmap('Spectral_r')

 # create a color palette, mapped to these values
 cmap = matplotlib.cm.Reds
 mini=min(secValue_DS)
 maxi=max(secValue_DS)
 norm = matplotlib.colors.Normalize(vmin=mini, vmax=maxi)
 colors = [cmap(norm(value)) for value in secValue_DS]

 ax = squarify.plot(sizes=secValue_DS, label=secLB_DS, color=cm(col), alpha=1.0)

 ax.set_title("Value : Industry/Sector") 
 plt.show()
 return pd.DataFrame({"lb":secLB_DS,"value":secValue_DS})


try:
    #ลำดับมีผลพยายามโหลดที่ ใน folder ที่กำลัง Implement ก่อน
    from starfishX import config as con 
    #print("Dev")
except:
    #print("Debug")
    import starfishX.config as con

prefix = "starfishX."

if(con.__mode__=="debug"):
    prefix = ""


exec("from "+prefix+"npl_fn import *")

from tensorflow.keras.preprocessing.text import Tokenizer
from tensorflow.keras.models import load_model

import json

def sentimentAnalysis(content,path=""):
    #Load the previously saved weights

    #drive.mount('/content/drive')

    weight_path = path+"sa_model.h5"
    #weight_path1 = os.path.dirname(weight_path)
    #print(weight_path1)
    model = create_model()
    #print(model.summary())
    model.load_weights(weight_path)
    
    with open(path+'dictionary.json', 'r') as dictionary_file:
      dictionary = json.load(dictionary_file)
    
    tokenizer = Tokenizer(num_words=2200)
    tokenizer.word_index = dictionary  #tokenizer.word_index
    setTokenizer(tokenizer)
    
    tw = pre_process_txt(content)
    prediction = model.predict(tw)
    result = trans_result(prediction)
    return result


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt 

from datetime import datetime as dt

from tvDatafeed import TvDatafeed, Interval

import seaborn as sns

def monthlyReturn(symbol,month,engine,market='set',plot=True):
    df = engine.tv.get_hist(symbol=symbol,exchange=market,interval=Interval.in_monthly,n_bars=month)
    
    ######
    df['Date'] = pd.to_datetime(df.index)
    df['Date'] = df['Date'].dt.strftime('%Y-%m-%d')
    
    df.Date = pd.to_datetime(df.Date)
    df['month'] = df.Date.dt.month.values
    df['year'] = df.Date.dt.year.values
    
    df['pct_change_'] = df.close.pct_change()
    df['pct_change_m'] = df['pct_change_'].shift(-1)
    
    df.drop(index=df.index[-1], 
        axis=0, 
        inplace=True)
    
    df = df.dropna()
    
    
    df['month_win'] = np.where(df.pct_change_m>=0,1,0)
    
    if(plot):
      df.groupby(df.month)['month_win'].sum().plot(kind='bar',title='monthly win')
      plt.show()
      df.groupby(df.index.month)['pct_change_m'].sum().plot(kind='bar',title='total monthly return')
        
  
      sns.set()  
      df = df.set_index('Date') 
      fig, ax = plt.subplots(figsize=(10,10))
      
      #rdgn = sns.diverging_palette(h_neg=130, h_pos=10, s=99, l=55, sep=3, as_cmap=True)
      season = df.pivot("month", "year", "pct_change_m")
      ax = sns.heatmap(season,center=0.00,cmap="PiYG",ax=ax)
      plt.title("Heatmap Apple Return")
      plt.show()  

    return df

def isset(nameVar):
    return nameVar in globals()

def rankWithRange(data,minScope,maxScope):
    basket = data.sort_values([data.columns[0]],ascending=False)

    basket['RANK'] = list(range(len(basket), 0,-1))
    maxRank = basket['RANK'].max()
    minRank = basket['RANK'].min()
    maxScope = maxScope
    minScope = minScope
    S = (minScope-maxScope)/(minRank-maxRank)
    Int = minScope - S*minRank
    basket['RS_Rank'] = basket['RANK']*S+Int  
    
    return basket

class HistStockPrice():
  def __init__(self):    
     self.tv = TvDatafeed()
    
  def days_between(self,d1, d2):
      d1 = dt.strptime(d1, "%Y-%m-%d")
      d2 = dt.strptime(d2, "%Y-%m-%d")
      return abs((d2 - d1).days)

  def getPrice(self,symbol,start,stop='',exchange='set'):
    
      date_now = dt.today().strftime("%Y-%m-%d") 
    
      if(stop==''):
        stop = date_now
    
      k = self.days_between(start, date_now)
    
      df = self.tv.get_hist(symbol=symbol,exchange=exchange,interval=Interval.in_daily,n_bars=k)
    
      df['Date'] = pd.to_datetime(df.index)
      df['Date'] = df['Date'].dt.strftime('%Y-%m-%d')
      df['Date'] = pd.to_datetime(df['Date'])

      df = df[['Date','close']]
      df = df.set_index('Date')
      df.columns = [symbol]
    
      df = df[(df.index>=start) & (df.index<=stop)]
      return df


#IPO_stat
#IPO Performance


import requests
import pandas as pd
import numpy as np
from bs4 import BeautifulSoup
import json

def ipoStat(start,end): 
 start = str(start)
 end = str(end)   
 '''
 สถิติหุ้น IPO
 start (string)  : ช่วงเวลาเริ่มต้น ใส่เป็นปี คศ. เช่น 2000
 end   (string)  : ช่วงเวลาเริ่มต้น ใส่เป็นปี คศ. เช่น 2000
 '''   
 #ipo-performance
 url = 'https://api.settrade.com/api/ipo/highlight/list?market=&industry=&type=&faId=&fromYear='+start+'&toYear='+end+'&language=th'

 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
 r = requests.get(url,headers=headers, timeout=5)
 soup = BeautifulSoup(r.content, "lxml")  

 #####
 JSONData = soup.find_all('p')[0].text

 #####
 storedata = json.loads(JSONData)
 ipoDS = (storedata['ipoHighlights'])
 #####

 symbolDS = []
 marketDS = []
 industryDS = []
 firstTradeDateDS = []
 ipoPriceDS = []
 financialAdvisorsDS = []
 firstTradePriceCloseDS = []
 firstTradePriceOpenDS = []
 ipoMarketCapDS = []
 ipoPEDS = []
 ipoPBVDS = []
 for i in ipoDS:
   symbol = i['symbol']
   market = i['market']
   industry = i['industry']
   firstTradeDate = i['firstTradeDate'] 
   firstTradeDate = pd.to_datetime(firstTradeDate.split('T')[0])
   ipoPrice = float(i['ipoPrice'])
   firstTradePriceClose = float(i['firstTradePrice']['close'])
   firstTradePriceOpen = float(i['firstTradePrice']['open'])
   
 
   ipoMarketCap = float(i['ipoStatistic']['marketCap'])
   try: 
    ipoPE = float(i['ipoStatistic']['pe'])
   except:
    ipoPE = np.NaN
    
   try: 
    ipoPBV = float(i['ipoStatistic']['pbv']) 
   except:
    ipoPBV = np.NaN
 
    
   try:
    financialAdvisors = np.array(i['financialAdvisors']).tolist()
    if(len(financialAdvisors)>1):
     t = financialAdvisors[0]
     for i in range(1,len(financialAdvisors)):
       t=t+'@'+(financialAdvisors[i])
     financialAdvisorsContext = t
    else:
     financialAdvisorsContext = financialAdvisors[0]
   except:
    financialAdvisorsContext = i['financialAdvisors'] 
    
   symbolDS.append(symbol)
   marketDS.append(market)
   industryDS.append(industry)
   firstTradeDateDS.append(firstTradeDate)
   ipoPriceDS.append(ipoPrice) 
   firstTradePriceCloseDS.append(firstTradePriceClose) 
   firstTradePriceOpenDS.append(firstTradePriceOpen) 

   financialAdvisorsDS.append(financialAdvisorsContext) 
   ipoMarketCapDS.append(ipoMarketCap)
   ipoPEDS.append(ipoPE)
   ipoPBVDS.append(ipoPBV)

  ############################  
 df = pd.DataFrame({'symbol':symbolDS,
                    'market':marketDS,
                    'industry':industryDS,
                    'ipoPrice':ipoPriceDS,
                    'ipoMarketCap':ipoMarketCapDS,
                    'ipoPE':ipoPEDS,
                    'ipoPBV':ipoPBVDS,
                    'firstTradeDate':firstTradeDateDS,
                    'firstTradePriceOpen':firstTradePriceOpenDS,
                    'firstTradePriceClose':firstTradePriceCloseDS,
                    'financialAdvisors':financialAdvisorsDS})

 ###########
 # 
 # 
 #
 df = df.sort_values('firstTradeDate')
 df = df.set_index('symbol')
 return df                   


try:
    #ลำดับมีผลพยายามโหลดที่ ใน folder ที่กำลัง Implement ก่อน
    #import config as con 
    from starfishX import config as con 
except:
    import starfishX.config as con

prefix = "starfishX."

if(con.__mode__=="debug"):
    prefix = ""

import numpy as np
exec("import starfishX as sx")

def checkServices():
  
 #1
 df = sx.loadHistData("aot",start="2019-01-01",end="2019-01-31")
 if(len(df)>0):
    print("loadHistData\t\t\t done.")

 #2
 df = sx.MonthlyReturn(["aot"],startDate="2016-01-01",endDate="2018-12-31",plot=False)
 if(len(df)>0):
    print("MonthlyReturn\t\t\t done.")   
    
 #3
 df = sx.listShareholders("aot")
 if(len(df)>0):
    print("listShareholders\t\t done.")

 #4    
 k = sx.findCASH("aot")
 if(not np.isnan(k)):
   print("findCASH\t\t\t done.")
 else:
   print("findCASH\t\t\t fail.")   

 #5
 df = sx.commonsizeBS("aot")
 if(len(df)>0):
    print("commonsizeBS\t\t\t done.")
    
 #6
 df = sx.commonsizeIS("aot")
 if(len(df)>0):
    print("commonsizeIS\t\t\t done.")  
    
 #7
 df = sx.getFinanceRatio(["scb","ktb"])
 if(len(df)>0):
    print("getFinanceRatio\t\t\t done.")
    
     
 #8
 df = sx.spiderCompare(["mint","cpn"],plot=False)
 if(len(df)>0):
    print("spiderCompare\t\t\t done.")     
    
 #9
 df = sx.getBalanceSheet("aot")
 if(len(df)>0):
    print("getBalanceSheet\t\t\t done.")  
    
 #10
 df = sx.getBalanceSheetCompare(["aot","mint"])
 if(len(df)>0):
    print("getBalanceSheetCompare\t\t done.") 
    
 #11
 df = sx.getMemberOfIndex(sx.indexMarket.SETHD)
 if(len(df)>0):
    print("getMemberOfIndex\t\t done.")  

 #12
 df = sx.listStockInSector("mint")
 if(len(df)>0):
    print("listStockInSector\t\t done.")   
    
 #13
 df = sx.getFundamentalInSector("delta",progress=False)
 if(len(df)>0):
    print("getFundamentalInSector\t\t done.")
    
 #14   
 symbols = ["ptt","cpf","mint","aav"]

 df = sx.loadHistData(symbols,start="2018-01-01")

 k = sx.MarkowitzProcess(df,alias=symbols,random=5000,plot=False)

 if(len(k)>0):
    print("MarkowitzProcess\t\t done.")

 #15
 df = sx.listSecurities(progress=False)
 if(len(df)>0):
    print("listSecurities\t\t\t done.")
    
   

 #16
 df = sx.findMarketCap(["aot","mint"],progress=False)
 if(len(df)>0):
    print("findMarketCap\t\t\t done.")
    
 #17
 df = sx.getIncomeStatement("aot")
 if(len(df)>0):
    print("getIncomeStatement\t\t done.")   
    
    
 #18
 df = sx.getIncomeStatementCompare(["aot","kce"])
 if(len(df)>0):
    print("getIncomeStatementCompare\t done.")    
    
 #19
 try:
  df,K = sx.DuPontROE("aot",year=2561)
  if(len(df)>0):
     print("DuPontROE\t\t\t done.")   
 except:
  if(type(df)==int):
    if(df==0):
     print("DuPontROE\t\t\t fail.") 
    
 #20
 df = sx.InterestBearingDebt("mint",viewlog=False)
 if(len(df)>0):
    print("InterestBearingDebt\t\t done.")   
    
 #21
 df = sx.DA("kce")
 if(len(df)>0):
    print("DA\t\t\t\t done.")     
    
 #22
 df = sx.getEVEBITDA("kce",year=2561)
 if(len(df)>0):
    print("getEVEBITDA\t\t\t done.")  
    
 #23
 try:
  df = sx.analyseEVEBITDA(["kce"],yearstart=2561,yearend=2562,viewlog=False)
  if(len(df)>0):
    print("analyseEVEBITDA\t\t\t done.")  
 except:
  print("analyseEVEBITDA\t\t\t fail.") 

 #24
 df = sx.morningstarGetfn("mint",sx.MStar.EarningsPerShare,printProgress=False)
 if(len(df)>0):
    print("morningstarGetfn\t\t done.")   
    
 #25
 df = sx.DividendDiscountModel("mint",year=2561,k=0.12)
 if(len(df)>0):
    print("DividendDiscountModel\t\t done.") 
    
 #26
 k = sx.governmentBond10year()
 if(type(k)==int):
   if(k==0):  
    print("governmentBond10year\t\t fail.")  
 if(type(k)==float):
    print("governmentBond10year\t\t done.")
    
 #27
 df = sx.beta("mint")
 try:
  if(len(df)>0):
    print("beta\t\t\t\t done.")  
 except:
  if(df==0):
    print("beta\t\t\t\t fail.") 

 #28
 Nday = 40
 rp = sx.linregressIndicator(["ptt","aot"],periodDay=Nday,viewlog=False,viewplot=False)
 try:
   if(len(rp)>0):
     print("linregressIndicator\t\t done.")
 except:
   if(type(rp)==int):
     print("linregressIndicator\t\t fail.")
 
 #29
 rp = sx.marketview(sx.indexMarket.SET,start="2018-01-01",viewplot=False)
 try:
   if(len(rp)>0):
     print("marketview\t\t\t done.")
 except:
   if(type(rp)==int):
     print("marketview\t\t\t fail.")

 #30
 rp = sx.marketViewForeignTrade(sx.indexMarket.SET,start="2018-01-01",viewplot=False)
 try:
   if(len(rp)>0):
     print("marketViewForeignTrade\t\t done.")
 except:
   if(type(rp)==int):
     print("marketViewForeignTrade\t\t fail.")       

 #31
 df = sx.loadHistData("aot",start="2019-01-01",Volume=True)
 symbol,dataset = sx.AnomalyDetection(df,contamination=0.01,Lastday=3)
 try:
  if(type(symbol)==str):
   print("AnomalyDetectionVolume\t\t done.")
 except:
  print("AnomalyDetectionVolume\t\t done.")      

 
 #32
 k = sx.thaibma("2017-01-01",bondtype="")
 if(type(k)==int):
   if(k==0):  
    print("thaibma\t\t\t\t fail.")  
 if(type(k)==float):
    print("thaibma\t\t\t\t done.")


 #33
 marketType = sx.indexMarket.SET
 BondType = sx.BondType.USTreasury10Year
 df = sx.EarningYieldGap(marketType,BondType,start="2019-01-01",viewlog=False,viewplot=False)
 try:
   if(len(df)>0):
       print("EarningYieldGap\t\t\t done.")
 except:
       print("EarningYieldGap\t\t\t fail.")     


 print("complete..")    


import starfishX as sx 

import requests
from bs4 import BeautifulSoup

import pandas as pd
import numpy as np

from starfishX.peterlynch.condition import IndicatorIndustrial as ind

def getListStockInSector(sample_symbol):
    '''
    sample_symbol : (str) ตัวอย่างหุ้นในอุตสาหกรรม
    '''
    return sx.listStockInSector(sample_symbol)


def getYoY(sample_symbol,indicator=ind.NetProfit):
   '''
   str หรือ DataFrame ก็ได้ หากเป็น DataFrame จะใช้ Col ที่ 1
   '''
   if(type(sample_symbol)==str):
    symbol_ds = sample_symbol.upper()   
    symbol_ds,h1,v1,h2,v2 =  getYoYProcess(symbol_ds,indicator) 
    df = pd.DataFrame({"YearT":[h1],"ValueT":[v1],"YearT-1":[h2],"ValueT-1":[v2]},index=[symbol_ds])
    return df 
      

   if(type(sample_symbol)==list): 
     sample_symbol = pd.DataFrame({"symbol":sample_symbol},index=sample_symbol)  
     sample_symbol = sample_symbol[sample_symbol.columns[0]].tolist()
   elif(type(sample_symbol)==pd.core.frame.DataFrame):  
     sample_symbol = sample_symbol[sample_symbol.columns[0]].tolist() 
   
   symbol_ds = [] 
   h1_ds = [] 
   v1_ds = []
   h2_ds = []
   v2_ds = []
     
   print("Processing..",end="")
   for i in sample_symbol:
       i = i.upper()  
       print(i+".",end="")
       s,h1,v1,h2,v2 = getYoYProcess(i,indicator)
       symbol_ds.append(s)
       h1_ds.append(h1)
       v1_ds.append(v1)
 
       h2_ds.append(h2)
       v2_ds.append(v2)

       df = pd.DataFrame({"YearT":h1_ds,"ValueT":v1_ds,"YearT-1":h2_ds,"ValueT-1":v2_ds},index=symbol_ds)
   print(".End.",end="")  
   return df


def getYoYProcess(symbol,indicator): 
 #symbol = "awc"
 symboltmp = symbol.replace("&","%26") 
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
 url = "https://www.set.or.th/set/factsheet.do?symbol="+symboltmp+"&ssoPageId=3&language=th&country=TH"
 r = requests.post(url,headers=headers, timeout=5)
 soup = BeautifulSoup(r.content, "lxml")  

 ct = soup.find_all("table",{"class":"table-factsheet-padding0"})  

 if(len(ct)==0):
    return symbol,np.NaN,np.NaN,np.NaN,np.NaN

 #### หา index #### content อยู่ช่องไหน
 i_index = 0
 for i in ct:
  if("งบกำไรขาดทุนเบ็ดเสร็จ (ลบ.)" in i.text):
      m_index = i_index
      break
  i_index+=1

 ##########
 tr = ct[m_index].find_all("tr")
 
 ##########
 noMeet = 0
 for i in tr:
    td = i.find_all("td")
    try:
      if("งบกำไรขาดทุนเบ็ดเสร็จ (ลบ.)" in td[0].text):
        h1 = str(td[1].text).strip() 
        h1 = h1.replace("\xa0","")
        h1 = h1.replace(" ","")
        
        h2 = str(td[2].text).strip()
        h2 = h2.replace("\xa0","")
        h2 = h2.replace(" ","")
    except:
        h1 = np.NaN
        h2 = np.NaN
        v1 = np.NaN 
        v2 = np.NaN 
    
  
    if(indicator.value=="NetProfit"):
     if(("รายได้จากการลงทุนสุทธิ" in td[0].text) or ("กำไร(ขาดทุน)สุทธิ" in td[0].text)):
        noMeet = 1
        try: 
         v1 = float(td[1].text.replace(",",""))
         v2 = float(td[2].text.replace(",",""))   
        except:
         h1 = np.NaN
         h2 = np.NaN
         v1 = np.NaN 
         v2 = np.NaN 
    
    
    if(indicator.value=="Sales"):
      if(("รายได้จากเงินลงทุน" in td[0].text) or ("ยอดขายสุทธิ" in td[0].text) 
                                      or ("รายได้ดอกเบี้ยและเงินปันผลสุทธิ" in td[0].text) 
                                      or ("รายได้ค่านายหน้า" in td[0].text)   ): 
        noMeet = 1
        try:    
         v1 = float(td[1].text.replace(",",""))
         v2 = float(td[2].text.replace(",",""))  
        except:         
         h1 = np.NaN
         h2 = np.NaN
         v1 = np.NaN 
         v2 = np.NaN  

 if(noMeet==0):
   h1 = np.NaN
   h2 = np.NaN
   v1 = np.NaN 
   v2 = np.NaN 
 
 return symbol,h1,v1,h2,v2   


#listSecurities.py
import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np


def getMarketCapWithHist(symbol):
 '''
 symbol : str ชื่อย่อของหุ้น
 '''  
 #ป้องกันพวก &
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
 mrkCap = np.NAN
 symbolK = symbol.replace('&', '%26')
 r = requests.post("https://www.set.or.th/set/factsheet.do?symbol="+symbolK+"&ssoPageId=3&language=th&country=TH",headers=headers)
 r.content
 soup = BeautifulSoup(r.content, "lxml")  

 g_data = soup.find_all("table", {"class" : "table-factsheet-padding0"})#[2].find_all("td")
 if(len(g_data)==0):
   print("ไม่พบข้อมูล")  
   return 0
 
 ##########################################
 if("มูลค่าหลักทรัพย์ตามราคาตลาด" in g_data[13].text):
    tr_ds = g_data[13].find_all("tr")
 elif("มูลค่าหลักทรัพย์ตามราคาตลาด" in g_data[14].text):
    tr_ds = g_data[14].find_all("tr")
 elif("มูลค่าหลักทรัพย์ตามราคาตลาด" in g_data[15].text):
    tr_ds = g_data[15].find_all("tr")
 elif("มูลค่าหลักทรัพย์ตามราคาตลาด" in g_data[12].text):
    tr_ds = g_data[12].find_all("tr")
 elif("มูลค่าหลักทรัพย์ตามราคาตลาด" in g_data[11].text):
    tr_ds = g_data[11].find_all("tr") 
 elif("มูลค่าหลักทรัพย์ตามราคาตลาด" in g_data[16].text):
    tr_ds = g_data[16].find_all("tr")
 elif("มูลค่าหลักทรัพย์ตามราคาตลาด" in g_data[10].text):
    tr_ds = g_data[10].find_all("tr")


 th_ds = []   
 data_ds = []
 for i in tr_ds:
    tddata = i.find_all("td")
    #print(tddata[0].text)
    if("ข้อมูลสถิติ" in tddata[0].text):
       for k in tddata:
          th_ds.append(k.text)
    if("มูลค่าหลักทรัพย์ตามราคาตลาด" in tddata[0].text):
       z = 0 
       for k in tddata:
          m = k.text  
          if(z>0):  
            if("-" in m):
                print("ไม่พบข้อมูล")
                return 0
            m = float(m.replace(",",""))
          data_ds.append(m)     
          z+=1
            
 ##########################################
 df = pd.DataFrame({"ind":th_ds,"ds":data_ds}).T
 df.columns = th_ds   
 df = df.drop(df[df["ข้อมูลสถิติ"]=="ข้อมูลสถิติ"].index)
 df = df.set_index("ข้อมูลสถิติ")
 return df 

def findMarketCapProcess(symbol):
    #ป้องกันพวก &
    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
    mrkCap = np.NAN
    symbolK = symbol.replace('&', '%26')
    r = requests.post("https://www.set.or.th/set/factsheet.do?symbol="+symbolK+"&ssoPageId=3&language=th&country=TH",headers=headers)
    r.content
    soup = BeautifulSoup(r.content, "lxml")  

    g_data = soup.find_all("table", {"class" : "table-factsheet-padding0"})#[2].find_all("td")
 
    if(len(g_data)==0):
       print("ไม่พบข้อมูล")  
       return 0,0

    #หาจาก table บน  
    k = g_data[2].find_all("tr")    
 
    #ตั้งไว้หา lable
    tLabel = k[0].find_all("td")
    indexdata = 0
    for i in tLabel:
     if("Market Cap" in i.text):
       mrkCap = k[1].find_all("td")[indexdata].text.strip().replace(",","")
       #print("Market Cap",symbol,mrkCap)
     indexdata +=1 
     #####################

    #ป้องกัน พวกข้อมูลที่เป็น '-'  
    try:
      mrkCap = np.float(mrkCap)  
    except:
      mrkCap = np.nan 

    return symbol,mrkCap

def findMarketCap(liststock,withHist=False,progress=True):
  '''
  คืนค่ามูลค่าตลาดของบริษัท : liststock เป็น str|list 
  str ถ้าต้องการคืนค่าครั้งละบริษัท เช่น sx.findMarketCap('aot') และสามารถใช้ withHist=True ได้
  list ถ้าต้องการคืนค่าครั้งละหลายๆ บริษัท เช่น sx.findMarketCap(['jmart','jmt']) แต่ไม่สามารถใช้ withHist=True ได้
  ''' 
  if(isinstance(liststock, str) and withHist==False):
    s,m = findMarketCapProcess(liststock)
    return m    
  
  if(withHist):
     if(isinstance(liststock, str)):
       return getMarketCapWithHist(liststock)
     else:
       print("not support format") 
       return 0

  ds_symbol = []
  ds_mrkcap = []  

  if(progress==True):
    print("Processing..",end="")
  pcnt = 0  
  cnt = len(liststock)
  for symbol in liststock:
    symbol,mrkCap = findMarketCapProcess(symbol)
    ds_symbol.append(symbol)
    ds_mrkcap.append(mrkCap)
    
    if(progress==True):
     if(pcnt==cnt-1):
      print(symbol,end="")
     else:
      print(symbol,end=",")
    
    pcnt +=1 
  df = pd.DataFrame({"symbol":ds_symbol,"MarketCap.":ds_mrkcap})
  df = df.set_index("symbol")

  if(progress==True):
   print(" Complete",end="")  
  return df  


import starfishX as sx
import pandas as pd
import numpy as np
from datetime import date
import datetime

import requests
from bs4 import BeautifulSoup

pd.options.display.float_format = '{:,.2f}'.format

def findName(symbol):
 symbol = symbol.replace("&","%26") 
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
 url = "https://www.set.or.th/set/factsheet.do?symbol="+symbol+"&ssoPageId=3&language=th&country=TH"
 r = requests.post(url,headers=headers, timeout=5)
 soup = BeautifulSoup(r.content, "lxml") 
 
 try:
  ct = soup.find_all("table",{"class":"table-factsheet-padding3"})  
  return ct[0].find_all("td")[1].text.replace(" ","")
 except:
  return 0

def getSumOfThePart(symbol,aliasNote):
 '''
    symbol:str ชื่อย่อของหุ้นแม่ที่ต้องการวิเคราะห์
    aliasNote:str ชื่อของ fileNote ที่ใช้สำหรับสร้าง csv ไฟล์ในการคำนวณ
 '''
 print("Process..",end="")
 aliasNote = aliasNote+".csv"
 try:      
  df_ = pd.read_csv(aliasNote)  
 except:
  bs = sx.listSecurities()
  df_ = sx.listShareholders(bs.index.tolist(),csv=aliasNote)
 
 print(".",end="")
   
 #preprocess
 keyword = findName(symbol)
 #print(keyword)
 keyword = keyword.replace("บริษัท","")
 keyword = keyword.replace("จำกัด","")
 keyword = keyword.replace("(มหาชน)","")
 df_["Name"] = df_["Name"].str.replace(" ","")
 ls_hold = df_[(df_["Name"].str.contains(keyword))]
 #print(ls_hold)
 #############    
 today = date.today()
 d1 = today.strftime("%Y-%m-%d")  
 d2 = today - datetime.timedelta(days=10)
 #############

 if(len(ls_hold)==0):
     print("no-data")
     return 0,0  

 p_hold = sx.loadHistData(ls_hold["Symbol"].tolist(),start=d2,end=d1)
 
 
 pnow = p_hold.T[[p_hold.T.columns[-1]]] #-1 คือวันสุดท้าย
 p = pnow[pnow.columns[0]].tolist()
 


 #q = np.array(ls_hold["Share"].str.replace(",","").astype(float)) 
 q = np.array(ls_hold["Share"]).astype(float)
 

 rp = pd.DataFrame({"price close":p,"name":ls_hold["Name"].values,"qty":q},index=ls_hold["Symbol"].tolist())

 rp["value"] = rp["price close"]*rp["qty"]
 #############

 
 
 p_hold = sx.loadHistData(symbol,start=d2,end=d1).values[-1] 


 sjmart = float(sx.getTotalShare(symbol))*1_000_000
 v = p_hold*sjmart
 p_hold = pd.DataFrame({"price close":p_hold,"qty":[sjmart],"value":v},index=[symbol.upper()])
 #############   
 print(".End.",end="")    
 return rp,p_hold


#getEVEBITDA.py
import pandas as pd

###################### ส่วนที่ใช้ Library ภายใน ###################
try:
    #ลำดับมีผลพยายามโหลดที่ ใน folder ที่กำลัง Implement ก่อน
    #import config as con 
    from starfishX import config as con
except:
    import starfishX.config as con

prefix = "starfishX."

if(con.__mode__=="debug"):
    prefix = ""   

# สำหรับ getMemberOfIndex    #starfishX Production #__init__ debug Mode
#from starfishX.findMarketCap import findMarketCap
#from starfishX.incomestatement import getIncomeStatement
#from starfishX.starfishXfn import getBalanceSheet
exec("from "+prefix+"findMarketCap import findMarketCap")
exec("from "+prefix+"incomestatement import getIncomeStatement")
exec("from "+prefix+"starfishXfn import getBalanceSheet")
###################### END ส่วนที่ใช้ Library ภายใน ###################


def analyseEVEBITDA(basket,yearstart,yearend,viewlog=True):
 """
    วิเคราะห์คุณภาพของ EV/EBITDA
 
    Returns ไม่มี
    
    arg1 (list): list ของตัวย่อของหุ้น เช่น ["aot","cpall","ptt"]
    
 """ 
 pcheck = 0    
 try:
   if(isinstance(basket.tolist(), list)):
      pass
 except:
   if(type(basket)==list):
      pass
   else:
      print("Data type miss match")  
      return 0  
 

 for s in basket: 
  if(viewlog==True): 
    print(s,end=" ")   
  _,k1 = getEVEBITDA(s,yearend)
  if(type(k1)==int):
    continue  
    
  _,k2 = getEVEBITDA(s,yearstart)
  if(type(k1)==int or type(k2)==int):
    continue
  
  if(viewlog==False): #เช็คแค่ตัวแรกของ basket
    return k1,k2

  marketCap61 = k1['marketCap'][0]
  marketCap60 = k2['marketCap'][0]

  TL61 = k1['TL'][0]
  TL60 = k2['TL'][0]

  cash61 = k1['cash'][0]
  cash60 = k2['cash'][0]

  ebitda61 = k1['ebitda'][0]
  ebitda60 = k2['ebitda'][0]
 
  ev61 = (marketCap61+TL61-cash61)  #EV
  ev60 = (marketCap60+TL60-cash60)  #EV
 
  evebitda61 = (ev61/ebitda61)  #EV/EBITDA
  evebitda60 = (ev60/ebitda60)  #EV/EBITDA

  p1 = False
  pctP1 = 0
  if(evebitda61<evebitda60):
    p1 = True
    pctP1 = (evebitda60 - evebitda61)/evebitda60
  else:
    pctP1 = (evebitda61 - evebitda60)/evebitda60

  pctP2 = 0
  p2 = False
  if( (ev61>ev60) ):
    p2 = True
    pctP2 = (ev61-ev60)/ev60
  else:
    pctP2 = (ev60-ev61)/ev60
    
    
  pctP3 = 0  
  p3 = False
  if(marketCap61>marketCap60):
    p3 = True 
    pctP3 = (marketCap61-marketCap60)/marketCap60
  else:
    pctP3 = (marketCap60-marketCap61)/marketCap60
    
    
  pctP4 = 0  
  p4 = False
  if(TL61<TL60):
    p4 = True 
    pctP4 = (TL60 - TL61)/TL60
  else:
    pctP4 = (TL61 - TL60)/TL60
    
  pctP5 = 0  
  p5 = False
  if(ebitda61>ebitda60):
    p5 = True 
    pctP5 = (ebitda61-ebitda60)/ebitda60
  else:
    pctP5 = (ebitda60-ebitda61)/ebitda60
    
 
  pctP1 = round(pctP1, 4)
  pctP2 = round(pctP2, 4)
  pctP3 = round(pctP3, 4)
  pctP4 = round(pctP4, 4)
  pctP5 = round(pctP5, 4)
         

  if(p1 and p2 and p3 and p4 and p5): #1evebitda 2ev 3marketcap 4TL 5ebitda
     print("***",end=" ")
  elif(p1 and p2 and p5): 
     print("**",end=" ")
  elif(p4 and p5): 
     print("*",end=" ")
 
  #สัญลักษณ์ต่างๆ 
  #http://xahlee.info/comp/unicode_arrows.html  ⬇⬆ ↑↓  ↑ ↓  ↑ ↓
  if(p1):
    if(pctP1>0.1):   
      print("EV/EBITDA⬇("+str(pctP1),end=") ")
    else:
      print("EV/EBITDA↓("+str(pctP1),end=") ")
  else:
    if(pctP1>0.1):
      print("EV/EBITDA⬆("+str(pctP1),end=") ")
    else:
      print("EV/EBITDA↑("+str(pctP1),end=") ")  
    
    
  if(p2):
    if(pctP2>0.1):
      print("EV⬆("+str(pctP2),end=") ")
    else:
      print("EV↑("+str(pctP2),end=") ")  
  else:
    if(pctP2>0.1):
      print("EV⬇("+str(pctP2),end=") ")
    else:
      print("EV↓("+str(pctP2),end=") ")
    
  if(p3):
    if(pctP3>0.1):
      print("Market Cap.⬆("+str(pctP3),end=") ")
    else:
      print("Market Cap.↑("+str(pctP3),end=") ")  
  else:
    if(pctP3>0.1):
      print("Market Cap.⬇("+str(pctP3),end=") ")
    else:
      print("Market Cap.↓("+str(pctP3),end=") ")
    
    
  if(p4):
    if(pctP4>0.1):
      print("Total Liabilities⬇("+str(pctP4),end=") ")
    else:
      print("Total Liabilities↓("+str(pctP4),end=") ")  
  else:
    if(pctP2>0.1):
      print("Total Liabilities⬆("+str(pctP4),end=") ")
    else:
      print("Total Liabilities↑("+str(pctP4),end=") ")
    
    
  if(p5):
    if(pctP5>0.1):
       print("EBITDA⬆("+str(pctP5)+")")
    else:
       print("EBITDA↑("+str(pctP5)+")") 
  else:
    if(pctP5>0.1):
       print("EBITDA⬇("+str(pctP5)+")")   
    else:
       print("EBITDA↓("+str(pctP5)+")") 
  

#############################################

def getEVEBIT(symbol,year,marketCapYTD=True):
  """
    Returns EV/EBIT,Log : จาก ( market cap. + หนี้สินรวม - เงินสด ) / EBIT
    
    ส่วน Log จะเป็น DataFrame ของข้อมูลดิบเอาไว้ตรวจสอบ
    
    arg1 (string): ตัวย่อของหุ้น
    
    arg2 (int): ปีพ.ศ. เช่น 2561 หรือ 2560

    * ไม่ support กลุ่ม bank และเช่าซื้อ
  """  
  if(type(year)==int):
    year=str(year)
    
  df1 = findMarketCap(symbol,withHist=True)
  if(type(df1)==int):
    return 0,0

  if(marketCapYTD==True):
   marketCap = df1.iloc[0][0] #เอา marketcap ล่าสุด
  else:  
   marketCap =  df1.getData(row="หลักทรัพย์",col=year)
  
  if(marketCap==0):
    return 0,0


    
  df2 = getIncomeStatement(symbol) 
  ebit = df2.getData(row="กำไรก่อนด/บ และภาษีเงินได้",col=year)
  if(ebit==0):
    #print("ไม่พบ ebitda,ไม่สามารถคำนวณได้") 
    return 0,0 
  if(ebit<0):
    print("EBIT ติดลบ,ไม่สามารถคำนวณได้") 
    return 0,0


  df3 = getBalanceSheet(symbol)
  cash = df3.getData(row="เงินสด",col=year)
  TL = df3.getData(row="รวมหนี้",col=year)

  bvebit  = (marketCap+TL-cash)/(ebit)
  rp = pd.DataFrame({"marketCap":[marketCap],"ebit":[ebit],"cash":[cash],"TL":[TL]})  
  return bvebit,rp




def getEVEBITDA(symbol,year):
  """
    Returns EV/EBITDA,Log : จาก ( market cap. + หนี้สินรวม - เงินสด ) / EBITDA
    
    ส่วน Log จะเป็น DataFrame ของข้อมูลดิบเอาไว้ตรวจสอบ
    
    arg1 (string): ตัวย่อของหุ้น
    
    arg2 (int): ปีพ.ศ. เช่น 2561 หรือ 2560

    * ไม่ support กลุ่ม bank และเช่าซื้อ
  """  
  if(type(year)==int):
    year=str(year)
    
  df1 = findMarketCap(symbol,withHist=True)
  if(type(df1)==int):
    return 0,0

  marketCap =  df1.getData(row="หลักทรัพย์",col=year)
  if(marketCap==0):
    return 0,0
    
  df2 = getIncomeStatement(symbol) 
  ebitda = df2.getData(row="ebitda",col=year)
  if(ebitda==0):
    #print("ไม่พบ ebitda,ไม่สามารถคำนวณได้") 
    return 0,0 
  if(ebitda<0):
    print("EBITDA ติดลบ,ไม่สามารถคำนวณได้") 
    return 0,0

  df3 = getBalanceSheet(symbol)
  cash = df3.getData(row="เงินสด",col=year)
  TL = df3.getData(row="รวมหนี้",col=year)

  bvebitda  = (marketCap+TL-cash)/ebitda
  rp = pd.DataFrame({"marketCap":[marketCap],"ebitda":[ebitda],"cash":[cash],"TL":[TL]})  
  return bvebitda,rp

#############################################
def getData(self,row,col):
    try:
       if(type(col)==int):
        self.columns[col]
    except:
       print("out of index") 
       return 0 

    ############ กรณี int ############ 
    if((type(row))==int and (type(col)==int)):
     d = self[self.columns[col]].iloc[row]
     if(type(d)==str):
        d = float(d.replace(",",""))
     
     return d   
        
    ############ กรณี string ############    
    if(type(row)==str):  
      rows = ""
      for i in self.index:
       if(row in str.lower(i)):
        rows = i
 
      cols = ""
      if(type(col)==int):
         cols = self.columns[col]
      elif(type(col)==str): 
       for i in self.columns:
        if(col in str.lower(i)[0:5]):
         cols = i
    
    if(rows==""):
      print("out of index,Rows") 
      return 0
    
    if(cols==""):
      print("out of index,Cols") 
      return 0  
    
    d = self[self.index==rows][cols]
    d = d.values[0]
    if(type(d)==str):
     d = float(d.replace(",",""))
        
    return d

#############################################
setattr(pd.core.frame.DataFrame, 'getData', getData)   
#############################################


#depreciationAndAmortisation.py
import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np

def DA(symbol):
 """
    Returns DA,Log : Depreciation และ Amortization  และ Log ของรายการ 
    
    arg1 (string): ตัวย่อของหุ้น

 """   
###### zone1 ดึงข้อมูล
 headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
 symbolMofi = symbol.replace("&","%26")
 #url = "https://www.set.or.th/set/companyhighlight.do?symbol="+symbolMofi
 url = "https://www.set.or.th/set/companyfinance.do?type=cashflow&symbol="+symbolMofi+"&language=th&country=TH"
 r = requests.get(url,headers)
 soup = BeautifulSoup(r.content, "lxml")
 #print(soup)
###### zone2 หาส่วนที่จะใช้
 #data = soup.find_all("table",{"class":"table table-hover table-info"})  
 data = soup.find_all("table",{"class":"table table-info-wrap"})   #19 สิงหาคม 2564 update
 #print(data)
 if(len(data)==0):
    print("ไม่พบข้อมูล2")
    return 0,0
 tr = data[2].find_all("tr") 

###### zone3 เตรียมดาต้ามาประมวลผล
 p = 0
 lableItem = []
 valueItem = []
    
 sublabelItem = []
 subvalueItem = []
 previousKey = ""   
 for td in tr:
   tddata = td.find_all("td")
   #for i in range(len(tddata)):
   try:
      if("ค่าเสื่อมราคาและค่าตัดจำหน่าย" in tddata[0].text):     #19 สิงหาคม 2564 update
         print(tddata[0].text,tddata[1].text)
         lableItem.append(tddata[0].text)
         valueItem.append(tddata[1].text)
         previousKey = tddata[1].text
         
      elif("\xa0" in tddata[0].text[1] and previousKey=="ค่าเสื่อมราคาและค่าตัดจำหน่าย"):
         sublabelItem.append(tddata[0].text) 
         subvalueItem.append(tddata[1].text)     
      
      else:
         previousKey = tddata[0].text         
   except:
     pass            
        
   
###### zone4 clear report    
 if(len(valueItem)==0):
    depre_and_m = np.NAN
    log = ""
 else:   
    depre_and_m = valueItem[0].replace(",","")
    depre_and_m = float(depre_and_m)   
    lableItem[0],valueItem[0]
    
    '''log = []
    for i in range(len(sublabelItem)):
      k1 = sublabelItem[i].replace("\xa0","")
      k2 = subvalueItem[i].replace(",","")
      log.append(k1+" "+k2)  '''
    
    log = lableItem[0] +" "+valueItem[0] #19 สิงหาคม 2564 update
     
###### zone5 return ผลลัพธ์
 return depre_and_m,log










import requests
from bs4 import BeautifulSoup
import pandas as pd

import datetime

from urllib import request, parse
import ssl
def getNVDRTrading(symbol,date):
    tmpd = date.split("-")
    year = tmpd[0]
    month = tmpd[1]
    day = tmpd[2]
    
    newdate_format = day+"/"+month+"/"+year
   
    ### ดึง soup ####
    #url = "https://www.set.or.th/set/nvdrbystock.do"
    
    #values = {"sort":"",
    #          "type" : "volume",
    #          "language" :"th",
    #          "country" : "TH",
    #          "date":newdate_format }

    #context = ssl._create_unverified_context()
    #data = parse.urlencode(values).encode()
    #req =  request.Request(url, data=data) # this will make the method "POST"
    #resp = request.urlopen(req,context=context)
    
    #soup = BeautifulSoup(resp.read() , "lxml")
    url = "https://www.set.or.th/set/nvdrbystock.do?sort=&type=volume&language=th&country=TH&date="+day+"%2F"+month+"%2F"+year
     
    #?sort=&type=volume&language=th&country=TH&date="+day+"%2F"+month+"%2F"+year
    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}
    r = requests.post(url,headers)
    soup = BeautifulSoup(r.content, "lxml")  
    
    ct = soup.find_all("table", {"class" : "table table-info table-striped"})[0].find_all("tbody")
    ct = ct[0].find_all("td")
    
    ### process text ###
    p = 0
    buy = 0
    sell = 0
    for i in range(len(ct)):
     #มีมา 6 แถว     
     if(ct[i].text==symbol):
        #print(ct[i].text)
        p = 1
     if(p==1):  
      buydata = ct[i+1].text.strip()
      selldata = ct[i+2].text.strip()
      if(buydata=="-"):
         buydata = "0"
      if(selldata=="-"):
         selldata = "0"
            
      buy = float(buydata.replace(",","")) #ปริมาณซื้อ
      sell = float((selldata).replace(",","")) #ปริมาณขาย
      break 

    return (symbol,buy,sell)


def getNVDRVolume(symbol,startdate):
 '''
 symbol (string) : สัญลักษณ์ของหุ้นไทย เช่น 'AOT' 
 startdate (string) : วันที่เริ่มดึงข้อมูล เช่น '2021-01-01'
 '''  
 symbol = symbol.upper()   
 symbolDS = []
 buyDS = []
 sellDS = []
 dateActive = []

 #### ส่วนของการสร้างวัน #####
 base = datetime.datetime.today()
 x=0
 date_list=[]
 while(True):
    d = base - datetime.timedelta(days=x)
    date_list.append(d)
    x+=1
    if(startdate==d.strftime("%Y-%m-%d")):  #วนจากวันที่ปัจจุบันมาวันที่เริ่มต้น
      break   
 #########################    
 
 print("process",end="")
 for d in date_list:
    date = (d.strftime("%Y-%m-%d"))
    try:
     symbol,buy,sell = getNVDRTrading(symbol,date)
     symbolDS.append(symbol)
     buyDS.append(buy)
     sellDS.append(sell)
     dateActive.append(d)   
     print(".",end="")
    except:
     print("-",end="")   
 print("end",end="")

 ##### คืนค่า dataframe #######
 df = pd.DataFrame({"symbol":symbolDS,"buy":buyDS,"sell":sellDS},index=dateActive)
 df.index = df.index.date #เทคนิค เอาเวลาออกจาก date คือตัด HH:MM:SS ออก
 return df

def reduceXtickfrequency(ax,n):
 '''
 ax : (plot)
 n  : จำนวนวันของแกน y 
 ax = df[["buy","sell"]].sort_index().plot(title="{} / NVDR Volume (Share)".format(symbol),
                                   kind="bar",figsize=(12,5),grid=True)
 sx.reduceXtickfrequency(ax,n=7)
 '''   
 ticks = ax.xaxis.get_ticklocs()
 ticklabels = [l.get_text() for l in ax.xaxis.get_ticklabels()]
 ax.xaxis.set_ticks(ticks[::n])
 ax.xaxis.set_ticklabels(ticklabels[::n])




#morningstarGetfn.py
import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np

 

########################################
from enum import Enum
class MStar(Enum):
 Revenue = "Revenue"
 GrossMargin = "Gross Margin %"
 OperatingIncome = "Operating Income"
 OperatingMargin = "Operating Margin %"
 NetIncome = "Net Income"
 EarningsPerShare = "Earnings Per Share"
 Dividends = "Dividends"
 PayoutRatio = "Payout Ratio %"
 Shares = "Shares"
 BookValuePerShare = "Book Value Per Share"
 OperatingCashFlow = "Operating Cash Flow"
 CapSpending = "Cap Spending"
 FreeCashFlow = "Free Cash Flow"
 FreeCashFlowPerShare = "Free Cash Flow Per Share"
 WorkingCapital = "Working Capital"
 
    
 TaxRate = "Tax Rate %"   
 NetMargin = "Net Margin %"
 AssetTurnoverAvg = "Asset Turnover (Average)"
 ROA = "Return on Assets %"
 FinancialLeverage = "Financial Leverage"
 ROE = "Return on Equity %"
 ROIC = "Return on Invested Capital %"
 InterestCoverage = "Interest Coverage"

    
######## main URL http://financials.morningstar.com/ratios/r.html?t=mint&region=THA&culture=en-us    
########################################   
def morningstarGetfn(symbol,Indicates,printProgress=False):
  """
    ดึงข้อมูลย้อนหลัง 10 ปีจาก morningstar
 
    Returns DataFrame
    
    arg1 (str): ตัวย่อของหุ้น เช่น "aot","cpall","ptt"

    arg2 (class MStar): ตัวคัดกรองที่ต้องการ เช่น ปันผลใช้ sx.MStar.Dividends
    
  """ 

  if(not isinstance(Indicates, MStar)):
    print("Data Type MStar Not Match")
    return 0
    
  IndicatesCheck = Indicates.value

  
  
  keywordlist = ["Tax Rate","Net Margin","Asset Turnover (Average)",
                "Return on Assets","Financial Leverage","Return on Equity",
                "Return on Invested Capital","Interest Coverage"]
  
  #if("Return on Equity" in Indicates):
  if any(s in IndicatesCheck for s in keywordlist):  
    #filename = "getKeyStatPart"
    #zoneU = True
    #print("zone2")
    return morningstarGetfnPart2inWeb(symbol,Indicates,printProgress)
  else:
    filename = "getFinancePart"
  
  Indicates = Indicates.value
  
  if(printProgress):
    print("Process.."+Indicates)
    
    
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
  symbolMofi = symbol.replace("&","%26")  
   # r = requests.get("http://financials.morningstar.com/finan/financials/getKeyStatPart.html?&callback=?&t=XBKK:KTC&region=tha&culture=en-US&cur=",headers=headers)


  r = requests.get("http://financials.morningstar.com/finan/financials/"+filename+".html?&callback=?&t=XBKK:"+symbol+"&region=tha&culture=en-US",headers=headers, timeout=5)
    
  #r.content
  soup = BeautifulSoup(r.content, "lxml")   
  
 
  data = soup.find_all("table")  
   
    
  IndicatesDS = []
  year_ds = []
 
  if(len(data)==0):
    print("ไม่พบข้อมูล")
    return 0
  
  meetData = False
  for i in data[0].find_all("tr"):    
    k = i.find_all("th")
    if(len(k)>=10):
      for j in k:
         datatmp = j.text.replace("-01","")
         datatmp = datatmp.replace("-02","") 
         datatmp = datatmp.replace("-03","")
         datatmp = datatmp.replace("-04","")    
         datatmp = datatmp.replace("-05","")
         datatmp = datatmp.replace("-06","")
         datatmp = datatmp.replace("-07","")
         datatmp = datatmp.replace("-08","")
         datatmp = datatmp.replace("-09","")
         datatmp = datatmp.replace("-10","")
         datatmp = datatmp.replace("-11","")
         datatmp = datatmp.replace("-12","")
         
          
          
         year_ds.append(datatmp)    
        
    if(Indicates in i.text and meetData==False):
      meetData = True  
      k = i.find_all("td")
      for j in k:   
         if("—" in j.text):
           IndicatesDS.append(np.NAN) 
         else:   
           if(j.text !=""): 
             m = j.text
             m = m.replace(",","")
             IndicatesDS.append(float(m))
  
  year_ds.pop(len(year_ds)-1)
  IndicatesDS.pop(len(IndicatesDS)-1) 
  year_ds.pop(0)

  year_ds_np = np.array(year_ds)
  year_ds = year_ds_np.astype(np.int)  
    
  df = pd.DataFrame({"year":year_ds,Indicates:IndicatesDS})  
  df = df.set_index("year")  
  return df    


def morningstarGetfnPart2inWeb(symbol,Indicates,printProgress=False):
  if(not isinstance(Indicates, MStar)):
    print("Data Type MStar Not Match")
    return 0
    
  Indicates = Indicates.value

  filename = "getKeyStatPart"
  
  if(printProgress):
    print("Process.."+Indicates)
    
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
  symbolMofi = symbol.replace("&","%26")  
   # r = requests.get("http://financials.morningstar.com/finan/financials/getKeyStatPart.html?&callback=?&t=XBKK:KTC&region=tha&culture=en-US&cur=",headers=headers)

  url = "http://financials.morningstar.com/finan/financials/"+filename+".html?&callback=?&t=XBKK:"+symbol+"&region=tha&culture=en-US"
  r = requests.get(url,headers=headers, timeout=5)
 
  soup = BeautifulSoup(r.content, "lxml")   
  

  dataz = soup.find_all("tbody")
  if(len(dataz)==0):
    print("ไม่พบข้อมูล")
    return 0
    
  data = dataz[1]#,{"class" : "classr_table1 text2 print97"})
  #data = data[1]
  #print(url) 

  #################################################
  IndicatesDS = []
  year_ds = []

  row = soup.find_all("thead")[1].find_all("th")
  cnt = 0  
  for i in row:
    if(cnt>0 and cnt <len(row)-1):
      year_ds.append(i.text.replace("-12",""))   
    cnt+=1
    
    
  for i in data:
    #if("table" in i.find_all("td")[0].text):
    rowDataKey = i.find_all("th")
    if(len(rowDataKey))>0:
       if(Indicates in rowDataKey[0].text):
         if(len(i.find_all("td")[0].find_all("div"))==0):
         #for(k in i):
         #  print(k)
          for k in i.find_all("td"):
            if("—" in k.text):
              IndicatesDS.append(np.NAN) 
            else:
              IndicatesDS.append(float(k.text))
            
  ################################################# 
  IndicatesDS.pop(len(IndicatesDS)-1)
  df = pd.DataFrame({"year":year_ds,Indicates:IndicatesDS})  
  df = df.set_index("year") 
  return df 


from pythainlp import word_tokenize
from pythainlp import Tokenizer as TokenizerNLP
from pythainlp.util import dict_trie
from pythainlp.corpus.common import thai_words

import pandas as pd
import numpy as np
import re

import json 

from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import LSTM,Dense, Dropout
from tensorflow.keras.layers import SpatialDropout1D
from tensorflow.keras.layers import Embedding

from tensorflow.keras.models import load_model

from tensorflow.keras.preprocessing.text import Tokenizer
from tensorflow.keras.preprocessing.sequence import pad_sequences

tokenizer = 0

embedding_vector_length = 60
vocab_size = 2300
maxlen = 60

def trans_result(r):
    r = int(r.round().item())
    if(r==0):
        return "บวก"
    else:
        return "ลบ"

def setTokenizer(s):
    global tokenizer
    tokenizer = s

def process_txt_init():
    words = ["ไทยออยล์","โควิด","โควิด19","โควิด-19","โคโรนา","D/E","P/E","P/BV","ออน์ไลน์","set50","set100",
             "เต็มมูลค่า","เทรดวอร์","เพิ่มทุน","มากกว่าคาด","Q1","Q2","Q3","Q4","B1","B2","B3",
             "มค","กพ","มีค","เมย","พค","มิย","กค","สค","กย","ตค","พย","ธค",
             "น้อยกว่าคาด","ตามคาด","ดีกว่าคาด","สูงกว่าคาด","ต่ำกว่าคาด","ผิดคาด","แย่กว่าคาด",
             "รีโนเวท","ศูนย์วิจัยกสิกร","กสิกรไทย","ผู้บริโภค","บริษัท","ทริส","ทริสเรทติ้ง","ซื้อหุ้นคืน",
             "กำไรพิเศษ","รีบาวด์","รีบาวนด์","รีบาวน์","สว่างหลังฝน","ฟ้าหลังฝน","ซีซั่น","ซีซัน","นกสกู๊ต",
             "วอลลุ่ม","ไตรมาส","ไตรมาศ","เชงเก้น","อีอีซี","ททท","ซินฟาตี้",
             "กองทุนการเงินระหว่างประเทศ","เวอร์จิน","อิควิตี้","รมต","แม็คโคร","ซีฟู้ด","ปปช","สคร","สตง","กสทช",
             "อีเมอร์จิ้ง","5G","ถุงมือยาง","ไฟเซอร์อิงค์","ดินถล่ม","อิจิฟุสะ","หนี้ครัวเรือน",
             "บาทแข็ง","บาทอ่อน","จิ้นผิง","สีจิ้นผิง","กิจการ","ไบออนเทค","เรมเดซิเวียร์","ออมนิแชแนล",
             "M&A","m&a","ควบรวมกิจการ","เหรินหมิน","อู่ฮั่น","ฮิวสตัน"]
    custom_words_list = set(thai_words())
    ## add multiple words
    custom_words_list.update(words)

    trie = dict_trie(dict_source=custom_words_list)
    custom_tokenizer = TokenizerNLP(custom_dict=trie, engine='newmm')
    return custom_tokenizer

def process_txt_split(txt):
    token = process_txt_init()
    k = token.word_tokenize(txt)
    k = [i for i in k if(i.replace(" ",""))]
    k = " ".join(k)
    return k

def process_txt_remove_float(txt):
    return re.sub("^\d+\s|\s\d+\s|\s\d+$|(\s\d+.\d+)+", " ", txt)

def process_txt_remove(txt):
    txt = txt.replace("TFRS 9","TFRS9")
    txt = txt.replace("“","")
    txt = txt.replace("“","")
    txt = txt.replace("”","")
    txt = txt.replace("”","")
    txt = txt.replace("‘","")
    txt = txt.replace("’","")
    txt = txt.replace("–","")
    txt = txt.replace("P/E","PE")
    txt = txt.replace("D/E","DE")
    txt = txt.replace("P/BV","PBV")
    txt = txt.replace("/"," ")
    txt = txt.replace("\"","")
    txt = txt.replace("\"","")
    txt = txt.replace("-","")
    txt = txt.replace(":","")
    txt = txt.replace("ฯ","")
    txt = txt.replace("(","")
    txt = txt.replace(")","")
    txt = txt.replace("'","")
    txt = txt.replace("%","")
    txt = txt.replace("m&a","ควบรวมกิจการ")
    txt = txt.replace("M&A","ควบรวมกิจการ")
    txt = txt.replace("โคโรน่า","โคโรนา")
    txt = txt.replace("วิตก","วิตกกังวล")
    txt = txt.replace("ล.","ล้าน")
    txt = txt.replace(".com","")
    txt = txt.replace(".net","")
    txt = txt.replace(".co.th","")
    txt = txt.replace(".","")
    txt = txt.replace("\xa0","")
    txt = process_txt_remove_float(txt)
    return txt

def pre_process_txt(txt):
    txt = process_txt_remove(txt) 
    txt = process_txt_split(txt)
    
    tw = tokenizer.texts_to_sequences([txt])
    tw = pad_sequences(tw,maxlen=maxlen)
    #prediction = int(model.predict(tw).round().item())
    #print(model.predict(tw),prediction)
    #print(prediction)
    return tw

def create_model():

 model = Sequential()
 model.add(Embedding(vocab_size, embedding_vector_length,     
                                     input_length=maxlen) )
 model.add(SpatialDropout1D(0.25))
 model.add(LSTM(128, dropout=0.5, recurrent_dropout=0.5))
 model.add(Dropout(0.2))
 model.add(Dense(512, activation='relu'))
 model.add(Dropout(0.2))
 model.add(Dense(1024, activation='relu'))
 model.add(Dropout(0.2))
 model.add(Dense(512, activation='relu'))
 model.add(Dropout(0.2))
 model.add(Dense(128, activation='relu'))
 model.add(Dropout(0.20))
 model.add(Dense(1, activation='relu'))

 model.compile(loss='binary_crossentropy',optimizer='adam', 
                           metrics=['accuracy'])
 #print(model.summary())
 return model


#indexMarket.py
from enum import Enum

class indexMarket(Enum):
 SET     = "SET"
 SET50   = "SET50"
 SET100  = "SET100"
 sSET    = "sSET"
 SETCLMV = "SETCLMV"
 SETHD   = "SETHD"
 SETTHSI = "SETTHSI"
 SETWB   = "SETWB"
 mai     = "mai" 


import requests
import pandas as pd
from datetime import datetime as dt
import numpy as np
import os

import mplfinance as fplt

from enum import Enum 
class timeFrame(Enum):
    day = '1d'
    week = '1wk'
    month = '1mo'

def loadHistData_v2(symbol,start,end,timeframe=timeFrame.day,typedata='history'):
  '''
  symbol :(str) 
  start :(str) วันที่เริ่มต้น YYYY-MM-DD คศ. เช่น 2020-01-01
  end :(str) วันที่สุดท้าย YYYY-MM-DD คศ. เช่น 2020-01-01
  typedata :(str) ตอนนี้มี 2 type 1.'history' และ 2. 'div'
  timeframe : class TimeFrame มี Day/Week และ Month
  
  ตัวอย่าง
  1.df = loadHistData(symbol='spali',start='2000-01-01',end='2021-01-09',timeframe=sx.timeFrame.week)
  2.df = loadHistData(symbol='spali',start='2000-01-01',end='2021-01-09',typedata='div')
  ''' 

  k = dt.strptime(start, '%Y-%m-%d')
  start = str(int(dt.timestamp(k)))
  k = dt.strptime(end, '%Y-%m-%d')
  end = str(int(dt.timestamp(k)))
 
  if('.US' in symbol):
    symbol_ = symbol.split('.')[0]
  else:
     symbol_ = symbol + '.BK'  

  filename = symbol+ '.csv'
    
  if(typedata=='history'): 
    if(timeframe==timeFrame.day):
       timeframe_ = '1d' 
    elif(timeframe==timeFrame.week):
       timeframe_ = '1wk' 
    elif(timeframe==timeFrame.month):    
       timeframe_ = '1mo' 
    
    url = 'https://query1.finance.yahoo.com/v7/finance/download/' + \
       symbol_+'?period1='+start+'&period2='+end+'&interval='+timeframe_+'&events=history'\
       '&includeAdjustedClose=true' 
    
  elif(typedata=='div'):
    url = 'https://query1.finance.yahoo.com/v7/finance/download/'+ \
       symbol_+'?period1='+start+'&period2='+end+'&interval=1d&events=div&includeAdjustedClose=true'
    
  #####
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   

  filename = symbol+'.BK'   
  try:
   r = requests.get(url,headers=headers)
   if('404 Not Found' in r.text):
     print(r.text)   
     return 0

   with open(filename, "wb") as code:
    code.write(r.content) 
  except:
    print('can\'t load data')
    return 0

  df = pd.read_csv(filename) 
  df = df.set_index('Date')
  df.index = pd.to_datetime(df.index)  
  os.remove(filename)

 
  return df.sort_index()


def plotCandle(self,filename=''):
  if(filename!=''):
    saving_params = dict(fname=filename,  bbox_inches='tight',pad_inches = 0.1)
    fplt.plot(
            self[['Open','High','Low','Close','Volume']],
            type='candle',
            style='charles',
            volume=True,
            figratio=(2,1),
            #title='',
            #ylabel='',
            axisoff=True,
            show_nontrading=False,
            savefig=saving_params
        )
  else:
    fplt.plot(
            self[['Open','High','Low','Close','Volume']],
            type='candle',
            style='charles',
            volume=True,
            figratio=(2,1),
            #title='',
            #ylabel='',
            axisoff=False,
            show_nontrading=False,
            #savefig=saving_params
        )
    
setattr(pd.core.frame.DataFrame, 'plotCandle', plotCandle)


# เอกสารจาก
#1.https://medium.com/swlh/black-scholes-algorithmic-delta-hedging-c2cdd42ce175 
#2.https://aaronschlegel.me/measure-sensitivity-derivatives-greeks-python.html


try:
    #ลำดับมีผลพยายามโหลดที่ ใน folder ที่กำลัง Implement ก่อน
    #import config as con 
    from starfishX import config as con 
except:
    import starfishX.config as con

prefix = "starfishX."

if(con.__mode__=="debug"):
    prefix = ""


#ฟังก์ชันสำหรับหา rho และ theta
import numpy as np
import scipy.stats as si
import sympy as sy

import math
from scipy.stats import norm

import pandas as pd
import numpy as np


from datetime import datetime

from datetime import datetime
import pandas as pd

def days_between(d1, d2):
    d1 = datetime.strptime(d1, "%Y-%m-%d")
    d2 = datetime.strptime(d2, "%Y-%m-%d")
    return abs((d2 - d1).days)


def EuropeanCallRolling(df_UnderRight,df_CallOption,ExerciseDate,strike_price,risk_free_rate):
    '''
    
    '''
    df = df_UnderRight
    dfw5 = df_CallOption 

    histUnderRight = []         #เก็บประวัติราคาแม่
    histPriceEuropeanCall = []  #เก็บประวัติราคาจาก EuropeanCall
    histRealPrice = []          #warrnt ที่เปลี่ยนบนกระดาน

    histDelta = [] 
    histgamma = []
    histVega = []

    #add Greek
    histRho = []
    histTheta = []

    histDate = []

    #หาราคาที่เปลี่ยนแปลงตามช่วงเวลา
    for i in dfw5.sort_index().index:
        date_of_warrant = str(i)[:10]
        #std นับถึงปัจจุบัน
        std = df[df.index<date_of_warrant][['Close']].std()[0]


        date_now = df[df.index<date_of_warrant][['Close']].index[-1]
        date_now = str(date_now)[:10]
        time = days_between(date_now,ExerciseDate)
        time_to_expiration = time/365
        #print(time) ใช้เวลาจาก dataframe จะมีวันเสาร์อาทิตย์หรือวันหยุดด้วยได้

        price_now = df[df.index<date_of_warrant][['Close']].iloc[-1][0]


        #log value change time step 
        #print(price_now,std,time,time_to_expiration,strike_price)
        callOption = EuropeanCall(asset_price=price_now, 
                                  asset_volatility=std,
                                  strike_price=strike_price,
                                  time_to_expiration=time_to_expiration, 
                                  risk_free_rate=risk_free_rate)

        priceW5Real = dfw5[dfw5.index==date_of_warrant]['close'][0]

        #print(callOption.price)  
        histDate.append(date_of_warrant)

        histUnderRight.append(price_now)  #ราคาแม่

        histRealPrice.append(priceW5Real) #ราคา w5 บนกระดาษจริงๆ    
        histPriceEuropeanCall.append(callOption.price) #ราคา w5 จาก EuropeanCall

        histDelta.append(callOption.delta)

        histgamma.append(callOption.gamma)

        histVega.append(callOption.vega)

        #update
        histTheta.append(callOption.theta)
        histRho.append(callOption.rho)



    histDate = pd.to_datetime(histDate)
    
    #########
    
    rp = pd.DataFrame({'UnderRight':histUnderRight,
                   'Actual':histRealPrice,
                   'EuropeanCall':histPriceEuropeanCall,
                   'delta':histDelta,
                   'gamma':histgamma,
                   'vega':histVega,
                   'theta':histTheta,
                   'rho':histRho},index=histDate)
    
    return rp




#เอกสารจาก https://medium.com/swlh/black-scholes-algorithmic-delta-hedging-c2cdd42ce175
class EuropeanCall:
    '''
    ``float asset_price : ราคาสินค้าอ้างอิง ``  
    ``float asset_volatility : ความผันผวนของราคาสินค้าอ้างอิง `` 
    ``float strike_price : ราคาใช้สิทธิ `` 
    ``float time_to_expiration : วันหมดอายุ (จำนวนวัน/365) ``
    ``float risk_free_rate : อัตราผลตอบแทนที่ไม่มีความเสี่ยง ``
    return : คืนค่าเป็น Option Price,Delta,Gamma,Vega,Theta และ Rho
    ตัวอย่าง :
    callOption = EuropeanCall(asset_price=4.94, asset_volatility=0.8093888412229526,strike_price=2,
                          time_to_expiration=2.9013698630136986, risk_free_rate=0.015)
    '''
    def call_delta(
        self, asset_price, asset_volatility, strike_price,
        time_to_expiration, risk_free_rate
            ):
        b = math.exp(-risk_free_rate*time_to_expiration)
        x1 = math.log(asset_price/(b*strike_price)) + .5*(asset_volatility*asset_volatility)*time_to_expiration
        x1 = x1/(asset_volatility*(time_to_expiration**.5))
        z1 = norm.cdf(x1)
        return z1

    def call_gamma(
        self, asset_price, asset_volatility, strike_price,
        time_to_expiration, risk_free_rate
            ):
        b = math.exp(-risk_free_rate*time_to_expiration)
        x1 = math.log(asset_price/(b*strike_price)) + .5*(asset_volatility*asset_volatility)*time_to_expiration
        x1 = x1/(asset_volatility*(time_to_expiration**.5))
        z1 = norm.cdf(x1)
        z2 = z1/(asset_price*asset_volatility*math.sqrt(time_to_expiration))
        return z2

    def call_vega(
        self, asset_price, asset_volatility, strike_price,
        time_to_expiration, risk_free_rate
            ):
        b = math.exp(-risk_free_rate*time_to_expiration)
        x1 = math.log(asset_price/(b*strike_price)) + .5*(asset_volatility*asset_volatility)*time_to_expiration
        x1 = x1/(asset_volatility*(time_to_expiration**.5))
        z1 = norm.cdf(x1)
        z2 = asset_price*z1*math.sqrt(time_to_expiration)
        return z2/100
    
    
    #เอกสารจาก https://aaronschlegel.me/measure-sensitivity-derivatives-greeks-python.html
    def rho_call(self, asset_price, asset_volatility, strike_price,time_to_expiration, risk_free_rate):
    
        d2 = (np.log(asset_price / strike_price) 
               + (risk_free_rate - 0.5 * asset_volatility ** 2) * time_to_expiration) / (asset_volatility * np.sqrt(time_to_expiration))
        rho = time_to_expiration * strike_price * np.exp(-risk_free_rate * time_to_expiration) * si.norm.cdf(d2, 0.0, 1.0)
        return rho
    
    #เอกสารจาก https://aaronschlegel.me/measure-sensitivity-derivatives-greeks-python.html
    def theta_call(self, asset_price, asset_volatility, strike_price,time_to_expiration, risk_free_rate):
        d1 = (np.log(asset_price / strike_price) + (risk_free_rate + 0.5 * asset_volatility ** 2) * time_to_expiration) / (asset_volatility * np.sqrt(time_to_expiration))
        d2 = (np.log(asset_price / strike_price) + (risk_free_rate - 0.5 * asset_volatility ** 2) * time_to_expiration) / (asset_volatility * np.sqrt(time_to_expiration))
        prob_density = 1 / np.sqrt(2 * np.pi) * np.exp(-d1 ** 2 * 0.5)
        theta = (-asset_volatility * asset_price * prob_density) / (2 * np.sqrt(time_to_expiration)) - risk_free_rate * strike_price * np.exp(-risk_free_rate * time_to_expiration) * si.norm.cdf(d2, 0.0, 1.0)
        return theta
    

    def call_price(
        self, asset_price, asset_volatility, strike_price,
        time_to_expiration, risk_free_rate
            ):
        b = math.exp(-risk_free_rate*time_to_expiration)
        x1 = math.log(asset_price/(b*strike_price)) + .5*(asset_volatility*asset_volatility)*time_to_expiration
        x1 = x1/(asset_volatility*(time_to_expiration**.5))
        z1 = norm.cdf(x1)
        z1 = z1*asset_price
        x2 = math.log(asset_price/(b*strike_price)) - .5*(asset_volatility*asset_volatility)*time_to_expiration
        x2 = x2/(asset_volatility*(time_to_expiration**.5))
        z2 = norm.cdf(x2)
        z2 = b*strike_price*z2
        return z1 - z2

    def __init__(
        self, asset_price, asset_volatility, strike_price,
        time_to_expiration, risk_free_rate
            ):
        self.asset_price = asset_price
        self.asset_volatility = asset_volatility
        self.strike_price = strike_price
        self.time_to_expiration = time_to_expiration
        self.risk_free_rate = risk_free_rate
        self.price = self.call_price(asset_price, asset_volatility, strike_price, time_to_expiration, risk_free_rate)
        self.delta = self.call_delta(asset_price, asset_volatility, strike_price, time_to_expiration, risk_free_rate)
        self.gamma = self.call_gamma(asset_price, asset_volatility, strike_price, time_to_expiration, risk_free_rate)
        self.vega = self.call_vega(asset_price, asset_volatility, strike_price, time_to_expiration, risk_free_rate)
        
        #update 
        #https://aaronschlegel.me/measure-sensitivity-derivatives-greeks-python.html
        self.rho = self.rho_call(asset_price, asset_volatility, strike_price, time_to_expiration, risk_free_rate)
        self.theta = self.theta_call(asset_price, asset_volatility, strike_price, time_to_expiration, risk_free_rate)




import matplotlib.pyplot as plt
from matplotlib.patches import Polygon
import numpy as np
import tensorflow as tf


def vector_plot(vecs, xlim, ylim, cols=["#1190FF", "#FF9A13"], alpha=1):
    plt.rc_context({'axes.edgecolor':'orange', 'xtick.color':'red', 'ytick.color':'red'})
    plt.axvline(x=0, color='k', zorder=0)
    plt.axhline(y=0, color='k', zorder=0)

    for i in range(len(vecs)):
        if (isinstance(alpha, list)):
            alpha_i = alpha[i]
        else:
            alpha_i = alpha
        x = np.concatenate([[0,0],vecs[i]])
        plt.quiver([x[0]],
                   [x[1]],
                   [x[2]],
                   [x[3]],
                   angles='xy', scale_units='xy', scale=1, color=cols[i],
                  alpha=alpha_i)
    plt.ylim(-xlim, xlim)
    plt.xlim(-ylim, ylim)
    plt.grid()


def plot_vector2d(vector2d, origin=[0, 0], **options):
    return plt.arrow(origin[0], origin[1], vector2d[0], vector2d[1],
              head_width=0.2, head_length=0.3, length_includes_head=True,
              **options)


def plot_transform(P_before, P_after, text_before, text_after, name, color=['#FF9A13', '#1190FF'], axis = [0, 5, 0, 4], arrows=False):
    if arrows:
        for vector_before, vector_after in zip(tf.transpose(P_before), tf.transpose(P_after)):
            plot_vector2d(vector_before, color="#FF9A13", linestyle="--")
            plot_vector2d(vector_after, color="#1190FF", linestyle="-")
    plt.rc_context({'axes.edgecolor':'orange', 'xtick.color':'red', 'ytick.color':'red'})
    plt.gca().add_artist(Polygon(tf.transpose(P_before), alpha=0.2))
    plt.gca().add_artist(Polygon(tf.transpose(P_after), alpha=0.3, color="#FF9A13"))
    plt.text(-.25, 1, text_before, size=18, color=color[1])
    plt.text(1.5, 0, text_after, size=18, color=color[0])
    plt.title(name, color='w')
    plt.axis(axis)
    plt.grid()


def evaluate(tensors):
    """Evaluates Tensor or EagerTensor to Numpy `ndarray`s.
    Args:
    tensors: Object of `Tensor` or EagerTensor`s; can be `list`, `tuple`,
      `namedtuple` or combinations thereof.
    Returns:
      ndarrays: Object with same structure as `tensors` except with `Tensor` or
        `EagerTensor`s replaced by Numpy `ndarray`s.
    """
    return tf.nest.pack_sequence_as(tensors,[t.numpy() if tf.is_tensor(t) else t for t in tf.nest.flatten(tensors)])


import requests
from bs4 import BeautifulSoup

import pandas as pd
import numpy as np 

def getAnalystForecast(symbol):
    symbol = symbol.replace("&","%26") 
    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}  
    url = "https://www.settrade.com/AnalystConsensus/C04_10_stock_saa_p1.jsp?txtSymbol="+symbol
    r = requests.post(url,headers=headers, timeout=5)
    soup = BeautifulSoup(r.content, "lxml")  

    ct = soup.find_all("table",{"class":"table table-info"})  
    if(len(ct)==0):
       return 0
 
    broker_ds = []
    targetPrice_ds = []
    tr = ct[0].find_all("tr") 
    for i in tr:
       td = i.find_all("td")
       if(len(td)>0):
          data = (td[1].text)
          price = (td[9].text)
          try:
            float(data)
          except:
            data = data.strip()
            if(data!="-"):  
              broker_ds.append(data)
              try:
                targetPrice_ds.append(float(price))
              except:
                targetPrice_ds.append(np.NaN)


    return pd.DataFrame({"Broker":broker_ds,"Target Price":targetPrice_ds})        


from enum import Enum
class condition(Enum):
 epsGrowth = "epsGrowth"
 listedShare = "listedShare"
 pe = "PE"
 peg = "PE/G"
 de = "de"
 interestBearingDebt = "interestBearingDebt" 
 cash = "cash"
 price = "price"
 equity = "equity"
 stockRepurchases = "stockRepurchases"
 cashPerShare = "cashPerShare"


class IndicatorIndustrial(Enum):
    NetProfit = "NetProfit"
    Sales = "Sales" 


#listSecurities.py
import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np

 

def findSector(symbol):
    #ป้องกันพวก &
    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
    mrkCap = np.NAN
    symbolK = symbol.replace('&', '%26')
    r = requests.get("https://www.set.or.th/set/factsheet.do?symbol="+symbolK+"&ssoPageId=3&language=th&country=TH",headers=headers)
    r.content
    soup = BeautifulSoup(r.content, "lxml")  

    g_data = soup.find_all("table", {"class" : "table-factsheet-padding0"})#[2].find_all("td")
 
    if(len(g_data)==0):
       print("ไม่พบข้อมูล")  
       return 0,0

    #หาจาก table บน  
    k = g_data[2].find_all("tr")    
 
    #ตั้งไว้หา lable
    tLabel = k[0].find_all("td")
    indexdata = 0
    for i in tLabel:
     if("Market Cap" in i.text):
       mrkCap = k[1].find_all("td")[indexdata].text.strip().replace(",","")
       #print("Market Cap",symbol,mrkCap)
     indexdata +=1 
     #####################
    return symbol,np.float(mrkCap)

